#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#ifdef __APPLE__
#include <strings.h>
#else
#include <string.h>
#include <memory.h>
#endif
#ifndef DEBUG
#include <qdos.h>
#else
#include <sys/types.h>
#endif

#include "../include/m4v.h"

#ifdef DEBUG
#define TESTFILE "test.m4v"
#else
#define TESTFILE "flp1_test.m4v"
#endif

#ifdef DEBUG
unsigned int screen[256][64];
#else
unsigned int *screen = (unsigned int *) 0x20000;
#endif

int main()
{
	int fd;
	int i, j, k, count;
	int cur_line = 0;
	unsigned char databyte;
	unsigned int *address;
	struct difference {
		unsigned char address;
		unsigned char value;
	} diffdat;
	unsigned int value;
	unsigned int line_buffer[64];

#ifndef DEBUG
	unsigned int *scr_line_ptrs[256];

	for (i = 0; i < 256; i++)
		scr_line_ptrs[i] = (unsigned int *)(screen + (64 * i));
#else
	unsigned int *scr_line_ptrs[256];

	for (i = 0; i < 256; i++)
		scr_line_ptrs[i] = screen[i];
#endif

	if ((fd = open(TESTFILE, O_RDONLY)) < 0)
	{
		fprintf(stderr, "Can't open %s\n", TESTFILE);
		return -1;
	}

	while((read(fd, &databyte, sizeof(unsigned char))) == sizeof(unsigned char))
	{
#ifdef DEBUG
		fprintf(stderr, "databyte is 0x%02x\n", databyte);
#endif
		if (isnewframe(databyte) == 1)
		{
			cur_line = 0;
#ifndef DEBUG
			mt_susjb(-1,1,NULL);
#else
			fprintf(stderr, "***************************** NEW BALLS PLEASE! ********************************\n");
#endif
		}
		else
		{
			if (cur_line == 256)
			{
				fprintf(stderr, "ERROR! got a frame with more than 256 lines!\n");
				return -1;
			}
		}

#ifdef DEBUG
		fprintf(stderr, "Line number: %d: \n", cur_line);
#endif

		if (isdiffline(databyte) == 1)
		{
			count = (databyte & 0x3f);
#ifdef DEBUG
			fprintf(stderr, "We have a differential line with %d changes.\n", count);
#endif
			for (i = 0; i < count; i++)
			{
				if (read(fd, &diffdat, sizeof(struct difference)) != sizeof(struct difference))
				{
					fprintf(stderr, "Bad read of difference block.\n");
					return -1;
				}
#ifdef DEBUG
				fprintf(stderr, "Change at line %d, block %d: %02x\n", cur_line, diffdat.address, diffdat.value);
#endif
				address = (unsigned int *)(scr_line_ptrs[cur_line] + diffdat.address);
				value = diffdat.value;
				*address = (value + (value >> 8));
			}
		}
		else
		{
#ifdef DEBUG
			fprintf(stderr, "Got a full scanline.\n");
#endif
			for (i = 0; i < 64; i++)
			{
				if (read(fd, &databyte, sizeof(unsigned char)) != sizeof(unsigned char))
				{
					fprintf(stderr, "Misread of fill line data.\n");
					return -1;
				}
				line_buffer[i] = (unsigned int)(((unsigned int) databyte) + (((unsigned int) databyte)>>8));
			}

			bcopy((char *)line_buffer, (char *)scr_line_ptrs[cur_line], 128);
		}

		cur_line++;
		databyte = 0x00;
	}
#ifdef DEBUG
	fprintf(stderr, "End of file, I think.\n");
#endif
	close(fd);
}
