/*
m4vencode.c

Encoder for Sinclair QL specific monochrome, mode 4 video codec.

Given the name of a directory containing QL Mode 4 screen dumps with names in
the form <directory name>-<frame number>.scr, e.g. "test-23.scr", combines
them into an encoded stream containing the differences between the current and
next frames.

The Sinclair QL Mode 4 screen is a matrix of pairs of bytes, each holding a
bitmap of 8 pixels. The first byte holds the "green" value and the second the
"red" value. There is no "blue" but the combination of "green" + "red" =
"white". We are only dealing with a bit mapped image (black or white pixels)
and hence we only need to store one of these bytes, halving the data. These
bytes containing 8 pixels (big-endian bit order) are called a "block" in this
program.

The QL's screen is arranged as 256 scan lines of 64 blocks wide, so we can
encode the location of the block in the scan line in 6 bits. Conveniently this
also means that we can encode the maximum number of changed blocks in a scan
line in 6 bits also, leaving the top two bits for other information, hence we
can encode each scan line thus:

<code byte>[<data>*]

If the code byte has the "start frame" bit set then it starts a new raster
frame, it terminates the previous frame immediately even if the current
raster line isn't the last. This means that a frame containing no differences
to the previous frame can be encoded as just one byte and also frames which
don't contain changes in the lower parts can be terminated prematurely.

If the code byte doesn't have the "difference" bit set then the following 64
bytes contain (in big-endian bit order) a bitmap of the scan line.

If the code byte has the "difference" bit set then the bottom 6 bits contains
the number of changed pixel blocks. The code byte is then followed by pairs of
bytes, the first os the address of the block along the line (0-64) followed by
the value. The number of blocks to be changed can be zero, in which case the
raster line doesn't need to be modified.

During the encoding if more than half the blocks in a raster line have changed
it's determined that it's more efficient to write a complete raster line
rather than a list of the changed blocks.

*/

main()
{
	char *filelist[65535];

	parse_args();

	get_filelist(filelist);

	encode_video(filelist);
}
